package com.gycsabesz.huntitdown.controller.item
{
	import com.gycsabesz.huntitdown.controller.plane.PlaneController;
	import com.gycsabesz.huntitdown.event.GameEvent;
	import com.gycsabesz.huntitdown.model.GameModel;
	import com.gycsabesz.huntitdown.model.plane.PlaneModel;
	import com.gycsabesz.huntitdown.view.GameView;
	import com.gycsabesz.huntitdown.view.Playground;
	import com.gycsabesz.huntitdown.view.hud.item.Item;
	
	import flash.display.MovieClip;
	import flash.events.TimerEvent;
	import flash.utils.Timer;

	/**
	 * Handles the generation of items.
	 */
	public class AbstractItemGenerator
	{
		protected var minimumInterval:Number;
		protected var maximumInterval:Number;
		
		protected var view:GameView;
		
		protected var planeController:PlaneController;
		
		protected var timer:Timer;
		
		/**
		 * Creates an item generator. Item will be generated after the minimumInterval time is elapsed but before the maximumInterval time is reached.
		 */
		public function AbstractItemGenerator(minimumInterval:Number, maximumInterval:Number, view:GameView, planeController:PlaneController)
		{
			setMinimumInterval(minimumInterval);
			setMaximumInterval(maximumInterval);
			
			this.view = view;
			this.planeController = planeController;
			
			startTimer();
			
			GameModel.getInstance().addEventListener(GameEvent.GAME_DEACTIVATED, gameModel_gameDeactivatedHandler);
			GameModel.getInstance().addEventListener(GameEvent.GAME_ACTIVATED, gameModel_gameActivatedHandler);
		}
		
		/**
		 * Stop this item generator when the game is stopped.
		 */
		protected function gameModel_gameDeactivatedHandler(e:GameEvent):void{
			if (timer != null){
				timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timer_completeHandler);
				timer.stop();
			}
		}
		
		/**
		 * Start this item generator when the game is started.
		 */
		protected function gameModel_gameActivatedHandler(e:GameEvent):void{
			startTimer();
		}
		
		/**
		 * Starts the timer of this generator. Timer will be started with a random delay between minimumInterval and maximumInterval.
		 */
		protected function startTimer():void{
			if (timer != null){
				timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timer_completeHandler);
				timer.stop();
			}
			
			// generate delay
			var delay:Number = minimumInterval * 1000 + Math.random() * (maximumInterval - minimumInterval) * 1000;
			timer = new Timer(delay, 1);
			timer.addEventListener(TimerEvent.TIMER_COMPLETE, timer_completeHandler);
			timer.start();
		}
		
		/**
		 * Callback method of the timer, which generates a new item.
		 */
		protected function timer_completeHandler(e:TimerEvent):void {
			var itemController:AbstractItemController = createItemControllerInstance();
			var itemView:Item = createItemViewInstance();
			
			itemView.setController(itemController);
			
			itemController.setView(itemView);
			itemController.setPlaneController(planeController);

			view.itemsContainer.addChild(itemView);
			itemView.x = view.playground.width;
			itemView.y = itemView.height / 2 + Math.random() * (view.playground.height - itemView.height);
			
			startTimer();
		}
		
		/**
		 * Set the time in seconds, which should be elapsed before a new item will be generated.
		 */
		public function setMinimumInterval(interval:Number):void{
			this.minimumInterval = interval;
		}

		/**
		 * Get the time in seconds, which should be elapsed before a new item will be generated.
		 */
		public function getMinimumInterval():Number{
			return minimumInterval;
		}
		
		/**
		 * Set the time in seconds, which mustn't be reached before a new item is generated.
		 */
		public function setMaximumInterval(interval:Number):void{
			this.maximumInterval = interval;
		}
		
		/**
		 * Get the time in seconds, which mustn't be reached before a new item is generated.
		 */
		public function getMaximumInterval():Number{
			return maximumInterval;
		}
		
		/**
		 * <p>Creates a new item.
		 * 
		 * <p>Should be implemented in subclasses.
		 */
		public function createItemViewInstance():Item{
			return null;
		}
		
		/**
		 * <p>Creates the controller of the new item.
		 * 
		 * <p>Should be implemented in subclasses.
		 */
		public function createItemControllerInstance():AbstractItemController{
			return null;
		}
	}
}